﻿package com.selenetan.ludumdare16 
{
	import com.selenetan.ludumdare16.Settings;
	import flash.display.Shape;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.events.Event;
	import flash.filters.GlowFilter;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;

	import com.greensock.TweenLite;
	import com.greensock.easing.*;
	
	import flash.display.Sprite;

	/**
	 * ...
	 * @author Selene
	 */
	public class Level
	{
		private var sprite:Sprite;
		private var player:Sprite;
		private var exit:ExitPortal;
		public var fog:Fog;
		private var shooter:Shooter;
		
		private var hintLayer:HintLayer;
		private var portalLayer:Sprite;
		private var playerLayer:Sprite;
		private var hudLayer:Sprite;
		private var menuLayer:MenuScreen;
		private var betweenLevelLayer:Sprite;
		
		public var portals:Array;
		private var currPortal:WarpPortal;
		
		private var difficulty:Number;
		private var levelsPlayed:Number;
		
		private var score:Number;
		private var scoreField:TextField;		
		private var flareField:TextField;
		
		private var levelMessage:TextField;

		private var hudTextFormat:TextFormat;
		
		private var soundManager:SoundManager;
		
		private var victoryFlash:Sprite;
		
		public function Level($sprite:Sprite, $player:Sprite, $exit:ExitPortal, $fog:Fog, $hintLayer:HintLayer, 
							  $shooter:Shooter, $menuLayer:MenuScreen, $soundManager:SoundManager)
		{
			sprite = $sprite;
			player = $player;
			exit = $exit;
			fog = $fog;
			hintLayer = $hintLayer;
			shooter = $shooter;
			menuLayer = $menuLayer;
			soundManager = $soundManager;
			
			portalLayer = new Sprite();
			playerLayer = new Sprite();
			hudLayer = new Sprite();
			
			setupHUD();
			setupVictoryFlash();
			setupBetweenLayer();
			setUpStage();
			
			portals = new Array();
			resetGame();
			menuLayer.addEventListener("startGame", startPlaying);
			
			sprite.stage.addEventListener(KeyboardEvent.KEY_DOWN, checkExitListener);
		}
		
		
		private function resetGame():void
		{
			trace("Level.resetGame()");
			levelsPlayed = 0;
			score = 0;
			
			shooter.reset();
			resetLevel();
		}
		
		private function resetLevel():void
		{
			//TweenLite.to(sprite, 0.4, { alpha: 1} );
			
			difficulty = levelsPlayed / 4 + 0.5;
			
			for (var i:int = 0; i < portals.length; ++i)
			{
				portals[i].removeEventListener(MouseEvent.MOUSE_DOWN, startWarp);
				portalLayer.removeChild(portals[i]);
			}
			
			exit.revealed = false;
			exit.removeEventListener(MouseEvent.MOUSE_DOWN, exitLevel);
			
			fog.reset();
			hintLayer.reset();
			createPortals();
			setStartAndExit();
			
			shooter.init();
			
			addScore(0);
			addFlare(0);
		}
		
		private function startPlaying(evt:Event = null):void
		{
			trace("Level.startPlaying()");
			
			resetGame();
			shooter.activate();
		}
		
		private function stopPlaying(evt:Event = null):void
		{
			shooter.deactivate();
		}
		
		private function setStartAndExit():void
		{
			var whichPortal:int;
			if (Math.random() < 0.5)
			{
				whichPortal = 0;
			}
			else
			{
				whichPortal = portals.length - 1;
			}
			
			player.x = portals[whichPortal].x;
			player.y = portals[whichPortal].y;
			portals[whichPortal].revealed = true;
			currPortal = portals[whichPortal];
			currPortal.hideGun();
			fog.revealCircle(player.x, player.y, Settings.REVEAL_RADIUS);
			currPortal.addEventListener(MouseEvent.MOUSE_DOWN, startWarp);


			// Set exit a certain distance from the start location
			
			var maxDist:Number = Math.sqrt(Settings.GAME_WIDTH * Settings.GAME_WIDTH + Settings.GAME_HEIGHT * Settings.GAME_HEIGHT) * 0.85;
			var desiredDist:Number = Settings.MAX_FLARE_DISTANCE * 1.1 * (difficulty);
			if (desiredDist > maxDist)
			{
				desiredDist = maxDist;
			}
			
			exit.x = -5;
			exit.y = -5;
			
			var dir:Number;
			var overlaps:Boolean = true;
			var numTries:int = 0;
			var allowedTries:int = 10;
			// Need to make sure that the exit is within the game bounds, and doesn't overlap any existing portals
			while (exit.x < 40 || exit.x > Settings.GAME_WIDTH - 40 || exit.y < 40 || exit.y > Settings.GAME_HEIGHT - 40 || overlaps)
			{
				dir = Math.random() * 2 * Math.PI;
				exit.x = player.x + desiredDist * Math.cos(dir);
				exit.y = player.y + desiredDist * Math.sin(dir);
				
				overlaps = false;
				for (var i:int = 0; i < portals.length; ++i)
				{
					if (exit.hitTestObject(portals[i]))
					{
						trace("found an overlap, re-randomizing...");
						overlaps = true;
					}
				}
				
				++numTries;
				if (numTries > allowedTries)
				{
					desiredDist -= 50;
					allowedTries += 5;
				}
			}
			
			portals.push(exit);
			portalLayer.addChild(exit);
		}
		
		
		private function createPortals():void
		{
			var portalSets:Array = new Array(4);
			
			portalSets[0] = [ [100, 100], [290,  90], [450,  60], [550,  60], [640, 120],
							  [600, 205], [450, 240], [290, 250], [155, 360], [100, 480],
							  [260, 530], [420, 525], [560, 480]
							];
			portalSets[1] = [ [90,  80], [300,  60], [460,  95], [670,  72],
							  [110, 260], [290, 270], [480, 275], [660, 250],
							  [130, 430], [320, 450], [450, 440], [680, 435],
							];
			/*portalSets[2] = [ [100,  75], [250,  75], [400,  75], [550,  75], [700,  75],
							  [100, 225], [250, 225], [400, 225], [550, 225], [700, 225],
							  [100, 375], [250, 375], [400, 375], [550, 375], [700, 375],
							  [100, 525], [250, 525], [400, 525], [550, 525], [700, 525]
							];*/
			portalSets[2] = [ [670,  75], [250,  75],                         [100, 100],
							              [270, 225], [380, 200], [530, 240], [700, 230],
							  [ 80, 375], [230, 350],                                    
							  [715, 505],             [360, 520], [550, 540], [120, 500]
							];
			portalSets[3] = [ [150,  75],             [400,  100], [570,  60],            
							  [100, 205], [260, 225], [420, 245],             [700, 210],
							  [130, 375], [250, 390],             [550, 350], [650, 325],
							              [220, 535], [460, 550], [560, 525], [720, 500]
							];
			
			var whichSet:Array = portalSets[int(Math.random()*4)];
			
			portals = new Array();
			var newPortal:WarpPortal;
			for (var i:int = 0; i < whichSet.length; ++i)
			{
				newPortal = new WarpPortal(whichSet[i][0], whichSet[i][1]);
				portals.push(newPortal);
				portalLayer.addChild(newPortal);
			}
		}
		
		
		private function addScore(amt:Number):void
		{
			score += amt;
			scoreField.text = "SCORE: " + score;
			scoreField.setTextFormat(hudTextFormat);
		}
		
		public function addFlare(amt:Number):void
		{
			shooter.numFlares += amt;
			flareField.text = "FLARES: " + int(shooter.numFlares);
			flareField.setTextFormat(hudTextFormat);
			
			// This needs to be < 1 because of the half-flares that aren't displayed
			if (shooter.numFlares < 1)
			{
				gameOver();
			}
		}
		
		private function setupHUD():void
		{
			hudTextFormat = new TextFormat("mainFont", 16, 0xFFFFFF, true);
			hudTextFormat.letterSpacing = 0.5;

			scoreField = new TextField();
			scoreField.text = "SCORE: " + score;
			scoreField.x = 10;
			scoreField.y = 10;
			scoreField.filters = [new GlowFilter(0xFFFFFF, 0.4, 3, 3)];
			scoreField.embedFonts = true;
			scoreField.selectable = false;
			scoreField.mouseEnabled = false;
			scoreField.setTextFormat(hudTextFormat);
			
			flareField = new TextField();
			flareField.text = "FLARES: " + shooter.numFlares;
			flareField.x = 10;
			flareField.y = 28;
			flareField.filters = [new GlowFilter(0xFFFFFF, 0.4, 3, 3)];
			flareField.embedFonts = true;
			flareField.selectable = false;
			flareField.mouseEnabled = false;
			flareField.setTextFormat(hudTextFormat);
			
			hudLayer.mouseChildren = false;
			hudLayer.mouseEnabled = false;
			hudLayer.addChild(scoreField);
			hudLayer.addChild(flareField);
		}
		
		private function setupVictoryFlash():void
		{
			victoryFlash = new Sprite();
			victoryFlash.graphics.lineStyle(2, 0xFFFFFF, 0.8, true);
			victoryFlash.graphics.drawCircle(0, 0, 50);
			victoryFlash.filters = [new GlowFilter(0xFFFFFF, 0.6)];
			victoryFlash.visible = false;
			victoryFlash.mouseEnabled = false;
			victoryFlash.name = "Sprite victoryFlash";
			
			hudLayer.addChild(victoryFlash);
		}
		
		private function setupBetweenLayer():void
		{
			betweenLevelLayer = new Sprite();
			var bg:Shape = new Shape();
			bg.graphics.beginFill(0x000000);
			bg.graphics.drawRect(0, 0, Settings.GAME_WIDTH, Settings.GAME_HEIGHT);
			betweenLevelLayer.addChild(bg);
			bg.name = "Shape Between Level BG";
			
			levelMessage = new TextField();
			var fmt:TextFormat = new TextFormat("mainFont", 36, 0x00FF00, true);
			fmt.letterSpacing = 0.5;
			levelMessage.defaultTextFormat = fmt;
			levelMessage.selectable = false;
			levelMessage.embedFonts = true;
			levelMessage.text = "LEVEL CLEARED";
			levelMessage.autoSize = TextFieldAutoSize.CENTER;
			levelMessage.x = (Settings.GAME_WIDTH - levelMessage.width) / 2.0;
			levelMessage.y = 250;
			levelMessage.mouseEnabled = false;
			levelMessage.name = "TextField LevelMessage";
			
			levelMessage.filters = [new GlowFilter(0x00FF00, 0.4, 5, 5)];
			
			betweenLevelLayer.name = "Sprite betweenLevelLayer";
			betweenLevelLayer.visible = false;
			
			betweenLevelLayer.addChild(levelMessage);
		}
		
		private function setUpStage():void
		{
			sprite.addChild(portalLayer);
			sprite.addChild(fog);
			sprite.addChild(playerLayer);
			sprite.addChild(hintLayer);
			sprite.addChild(hudLayer);
			sprite.addChild(betweenLevelLayer);
			betweenLevelLayer.visible = false;
			
			shooter.addElementsTo(playerLayer);
			playerLayer.addChild(player);
		}
		
		public function findAndRevealNearestPortal(x:Number, y:Number):void
		{
			// Check if a warp portal has been revealed
			var dist:Number;
			var minDist:Number = 1 << 30;
			var closestPortal:WarpPortal;
			var foundPortal:Boolean = false;
			var foundHidden:Boolean = false;
			var currPortal:WarpPortal;
			for (var i:int = 0; i < portals.length; ++i)
			{
				currPortal = portals[i];
				dist = (currPortal.x - x) * (currPortal.x - x) + (currPortal.y - y) * (currPortal.y - y);
				dist = Math.sqrt(dist);
				if (!currPortal.revealed) {
					if (dist < Settings.REVEAL_RADIUS + 5)
					{
						if (currPortal == exit)
						{
							trace("\nFound the exit!!");
							currPortal.addEventListener(MouseEvent.MOUSE_DOWN, exitLevel, false, 10);
						}
						else
						{
							currPortal.addEventListener(MouseEvent.MOUSE_DOWN, startWarp, false, 9);
						}
						fog.revealCircle(currPortal.x, currPortal.y, Settings.PORTAL_LIGHT_RADIUS);
						currPortal.revealed = true;
						addScore(Settings.SCORE_FIND_PORTAL);
						addFlare(Settings.FLARES_PER_PORTAL);
						foundPortal = true;
					}
					else if (dist < minDist)
					{
						minDist = dist;
						foundHidden = true;
					}
				}
			}
			
			if (!foundPortal && foundHidden)
			{
				soundManager.playEffect("ping");
				hintLayer.showHint(x, y, minDist);
			}
		}
		
		internal function startWarp(evt:MouseEvent):void
		{
			soundManager.playEffect("warp");
			evt.stopPropagation();
			warpToPortal(evt.target as Sprite);
		}
		
		internal function warpToPortal(portal:Sprite):void
		{
			trace("warpToPortal(" + portal + ")");
			
			currPortal.showGun();
			
			player.x = portal.x;
			player.y = portal.y;
			
			shooter.arrow.x = player.x;
			shooter.arrow.y = player.y;
			
			currPortal = portal as WarpPortal;
			currPortal.hideGun();
		}
		
		internal function exitLevel(evt:MouseEvent = null):void
		{
			exit.removeEventListener(MouseEvent.MOUSE_DOWN, exitLevel);
			soundManager.playEffect("explosion");
			addScore(Settings.SCORE_FIND_EXIT);
			if (evt) evt.stopPropagation();
			
			victoryFlash.visible = true;
			victoryFlash.x = exit.x;
			victoryFlash.y = exit.y;
			victoryFlash.scaleX = 0.1;
			victoryFlash.scaleY = 0.1;
			victoryFlash.alpha = 1;
			TweenLite.to(victoryFlash, 0.8, { scaleX: 2, scaleY: 2, alpha: 0, visible: false, ease: Bounce.easeInOut } );

			betweenLevelLayer.alpha = 0;
			betweenLevelLayer.visible = true;
			TweenLite.to(betweenLevelLayer, 0.7, { alpha: 1, delay: 1, onComplete: resetLevel } );
			TweenLite.to(betweenLevelLayer, 0.6, { alpha: 0, delay: 1.7, overwrite: false, onComplete: function():void { betweenLevelLayer.visible = false;} } );
			trace("You win!");
			
			++levelsPlayed;
			
			trace("Level score: " + score);
		}
		
		private function gameOver():void
		{
			trace("Game over, you lose!");
			stopPlaying();
			menuLayer.showGameOver(score);			
		}
		
		
		private function checkExitListener(evt:KeyboardEvent):void
		{
			trace("\nLevel number: " + levelsPlayed);
			trace("\n" + exit.toString() + " has MOUSE_DOWN listener? " + exit.hasEventListener(MouseEvent.MOUSE_DOWN));
			trace("    is revealed? " + exit.revealed);
			var i:int;
		}
		
	}

}