﻿package com.selenetan.ludumdare16 
{
	import com.greensock.TweenLite;
	
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.filters.GlowFilter;
	/**
	 * ...
	 * @author Selene
	 */
	public class Shooter
	{
		private var sprite:Sprite;
		
		private var player:Sprite;
		public var flare:Sprite;
		public var arrow:Sprite;
		public var strengthIndicator:Sprite;
		
		public var numFlares:Number;
		private var level:Level;
		
		public var throwStrength:Number;
		public var tick:Number;
		
		private var soundManager:SoundManager;

		public function Shooter($sprite:Sprite, $player:Sprite, $soundManager:SoundManager)
		{
			sprite = $sprite;
			player = $player;
			soundManager = $soundManager;

			numFlares = Settings.STARTING_FLARES;
			
			createArrow();
			createStrengthIndicator();
			createFlare();
		}
		
		public function reset():void
		{
			numFlares = Settings.STARTING_FLARES;
			init();
		}
		
		public function init():void
		{
			arrow.x = player.x;
			arrow.y = player.y;
			arrow.rotation = 0;
			
			strengthIndicator.x = player.x - 20;
			strengthIndicator.y = player.y;
			
			flare.x = player.x;
			flare.y = player.y;
			
			activate();
		}
		
		public function activate():void
		{
			sprite.addEventListener(MouseEvent.MOUSE_MOVE, updateAimArrow);
			sprite.addEventListener(MouseEvent.MOUSE_DOWN, showStrengthIndicator);
		}
		
		public function deactivate():void
		{
			sprite.removeEventListener(MouseEvent.MOUSE_MOVE, updateAimArrow);
			sprite.removeEventListener(MouseEvent.MOUSE_DOWN, showStrengthIndicator);
			sprite.removeEventListener(Event.ENTER_FRAME, updateStrengthIndicator);
			sprite.removeEventListener(MouseEvent.MOUSE_UP, shootFlare);
		}
		
		private function createArrow():void
		{
			arrow = new Sprite();
			arrow.graphics.lineStyle(2, 0xFFFF000, 0.8, true);
			arrow.graphics.moveTo(0, 3);
			arrow.graphics.lineTo(30, 3);
			arrow.graphics.lineTo(30, -3);
			arrow.graphics.lineTo(0, -3);			
			arrow.mouseEnabled = false;
			arrow.name = "Sprite arrow";
			
			arrow.filters = [new GlowFilter(0xFFFF00, 0.6, 6, 6, 2, 1)];
		}
		
		private function createStrengthIndicator():void
		{
			strengthIndicator = new Sprite();
			strengthIndicator.graphics.lineStyle(2, 0xFF8800, 0.8, true);
			strengthIndicator.graphics.drawRect(0, 0, 10, 40);
			strengthIndicator.rotation = 180;
			strengthIndicator.mouseEnabled = false;
			strengthIndicator.visible = false;
			strengthIndicator.name = "Sprite strengthIndicator";
			
			strengthIndicator.filters = [new GlowFilter(0xFF8800, 0.6, 6, 6, 2, 1)];
		}
		
		private function createFlare():void
		{
			flare = new Sprite();
			flare.graphics.lineStyle(1, 0xFF0000, 0.8, true);
			flare.graphics.drawCircle(0, 0, 4);
			flare.mouseEnabled = false;
			flare.visible = false;
			flare.name = "Sprite flare";
			
			flare.filters = [new GlowFilter(0xFF0000, 0.6, 6, 6, 2, 1)];
		}
		
		public function setLevel($level:Level):void
		{
			level = $level;
		}
		
		public function addElementsTo(sprite:Sprite):void
		{
			sprite.addChild(arrow);
			sprite.addChild(strengthIndicator);
			sprite.addChild(flare);
		}
		
		
		// ======  Aim, charge, and fire  =====
		
		public function updateAimArrow(evt:MouseEvent):void
		{
			var direction:Number = Math.atan2(sprite.mouseY - player.y, sprite.mouseX - player.x);
			
			arrow.rotation = direction * 180.0 / Math.PI;
		}
		
		public function updateStrengthIndicator(evt:Event):void
		{
			throwStrength = 1 - Math.abs(Math.cos(tick));
			strengthIndicator.scaleY = (throwStrength + 0.001);
			tick += 4 * Math.PI / 180;
		}
		
		public function showStrengthIndicator(evt:MouseEvent):void
		{
			trace("\nShooter.showStrengthIndicator()");
			trace("target: " + evt.target + " (" + evt.target.name + "), currentTarget: " + evt.currentTarget + " (" + evt.currentTarget.name + ")");
			trace("event phase: " + evt.eventPhase);
			
			strengthIndicator.visible = true;
			strengthIndicator.x = player.x - 20;
			strengthIndicator.y = player.y;
			
			tick = 0;
			sprite.removeEventListener(MouseEvent.MOUSE_DOWN, showStrengthIndicator);
			sprite.addEventListener(Event.ENTER_FRAME, updateStrengthIndicator);
			sprite.addEventListener(MouseEvent.MOUSE_UP, shootFlare);
		}
		
		private function shootFlare(evt:MouseEvent):void
		{
			soundManager.playEffect("shoot");
			
			sprite.removeEventListener(MouseEvent.MOUSE_MOVE, updateAimArrow);
			sprite.removeEventListener(Event.ENTER_FRAME, updateStrengthIndicator);
			sprite.removeEventListener(MouseEvent.MOUSE_UP, shootFlare);
			
			flare.visible = true;
			flare.x = player.x;
			flare.y = player.y;
			flare.scaleX = 1;
			flare.scaleY = 1;
			flare.alpha = 1;
			sprite.addChild(flare);
			
			var distance:Number = Settings.MAX_FLARE_DISTANCE * throwStrength;
			var xDist:Number = distance * Math.cos(arrow.rotation * Math.PI / 180);
			var yDist:Number = distance * Math.sin(arrow.rotation * Math.PI / 180);
			
			TweenLite.to(flare, throwStrength * 1.8, { x: flare.x + xDist, y: flare.y + yDist, onComplete: explodeFlare });
		}

		
		private function explodeFlare():void
		{
			soundManager.playEffect("flare");
			
			level.fog.revealCircle(flare.x, flare.y, Settings.REVEAL_RADIUS);
			TweenLite.to(flare, 0.8, { scaleX: 10, scaleY: 10, alpha: 0, visible: false, onComplete: function():void { sprite.removeChild(flare); } } );			
			
			level.findAndRevealNearestPortal(flare.x, flare.y);
			
			sprite.addEventListener(MouseEvent.MOUSE_MOVE, updateAimArrow);
			sprite.addEventListener(MouseEvent.MOUSE_DOWN, showStrengthIndicator);
			strengthIndicator.visible = false;
			
			level.addFlare( -1);
		}

	}

}