﻿package com.selenetan.ludumdare16 
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	import flash.filters.GlowFilter;
	import flash.events.MouseEvent;
	
	/**
	 * ...
	 * @author Selene
	 */
	public class SoundManager
	{
		
		private var bgMusic:Sound;
		private var bgMusicChannel:SoundChannel;

		private var pingSound:Sound;
		private var explosionSound:Sound;
		private var flareSound:Sound;
		private var warpSound:Sound;
		private var shootSound:Sound;
		
		private var muted:Boolean;
		
		private var toggleButton:Sprite;
		private var muteText:TextField;
		private var unmuteText:TextField;
		
		public function get muteButton():Sprite
		{
			return toggleButton;
		}
		
		public function SoundManager() 
		{
			pingSound = new Resources.pingSound();
			explosionSound = new Resources.explodeSound();
			shootSound = new Resources.shootSound();
			flareSound = new Resources.flareSound();
			warpSound = new Resources.warpSound();
			bgMusic = new Resources.bgMusic();
			
			createMuteButton();
			
			muted = false;
			startMusic();
		}
		
		public function playEffect(name:String):void
		{
			if (muted)
			{
				return;
			}
			
			switch(name)
			{
				case "warp": warpSound.play();
							 break;
				case "ping": pingSound.play();
							 break;
				case "flare": flareSound.play();
							 break;
				case "explosion": explosionSound.play();
							 break;
				case "shoot": shootSound.play();
							 break;
			}
		}
		
		public function isMuted():Boolean
		{
			return muted;
		}
		
		
		private function startMusic():void
		{
			bgMusicChannel = bgMusic.play(0, 5000);
		}
		
		private function stopMusic():void
		{
			bgMusicChannel.stop();
		}
		
		public function toggleMute(evt:Event=null):void
		{
			if (evt)
				evt.stopPropagation();
			
			if (muted)
			{
				startMusic();
				muteText.visible = true;
				unmuteText.visible = false;
			}
			else
			{
				stopMusic();
				muteText.visible = false;
				unmuteText.visible = true;
			}
			
			muted = !muted;
		}
		
		
		
		private function createMuteButton():void
		{
			trace("SoundManager.createMuteButton()");
			
			toggleButton = new Sprite();
			
			muteText = new TextField;
			muteText.text = "MUTE";
			muteText.selectable = false;
			muteText.mouseEnabled = false;
			muteText.embedFonts = true;
			muteText.defaultTextFormat = new TextFormat("mainFont", 12, 0xFFFFFF, true);
			muteText.setTextFormat(muteText.defaultTextFormat);
			muteText.autoSize = TextFieldAutoSize.CENTER;
			muteText.filters = [new GlowFilter(0xFFFFFF, 0.3, 3, 3)];
			
			unmuteText = new TextField;
			unmuteText.text = "UNMUTE";
			unmuteText.selectable = false;
			unmuteText.mouseEnabled = false;
			unmuteText.embedFonts = true;
			unmuteText.defaultTextFormat = muteText.defaultTextFormat;
			unmuteText.setTextFormat(unmuteText.defaultTextFormat);
			unmuteText.autoSize = TextFieldAutoSize.CENTER;
			unmuteText.filters = [new GlowFilter(0xFFFFFF, 0.3, 3, 3)];
			unmuteText.visible = false;
			
			toggleButton.graphics.beginFill(0x000000, 0.1);
			toggleButton.graphics.drawRect(0, 0, 40, 14);
			toggleButton.filters = [new GlowFilter(0xFFFFFF, 0.3, 3, 3)];
			
			toggleButton.addChild(muteText);
			toggleButton.addChild(unmuteText);
			
			trace("muteText.x: " + muteText.x);
			muteText.x = unmuteText.width - muteText.width;
			unmuteText.x = 0;
			trace("     after: " + muteText.x);
			
			toggleButton.x = Settings.GAME_WIDTH - toggleButton.width - 10;
			toggleButton.y = 10;
			
			toggleButton.addEventListener(MouseEvent.MOUSE_DOWN, toggleMute);
			toggleButton.addEventListener(MouseEvent.MOUSE_OVER, highlightButton);
			toggleButton.addEventListener(MouseEvent.MOUSE_OUT, unhighlightButton);
		}
		
		
		private function highlightButton(evt:Event = null):void
		{
			if (evt)
			{
				evt.currentTarget.filters = [new GlowFilter(0xFFFFFF, 0.4, 8, 8, 3)];
			}
		}
		
		private function unhighlightButton(evt:Event = null):void
		{
			if (evt)
			{
				evt.currentTarget.filters = [new GlowFilter(0xFFFFFF, 0.3, 3, 3)];
			}
		}
	}

}